"use strict";
"use-strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Square = require("./square.js");

var debug = require("debug")("MineBlown-board");

var Board = function () {
	function Board(height, width, minesIndices) {
		var _this = this;

		_classCallCheck(this, Board);

		this.height = height;
		this.width = width;
		this.minesIndices = minesIndices;
		this.minesCount = minesIndices.length;
		this.flagsIndices = [];
		this.flagsCount = 0;

		this.totalSquares = width * height;
		this.discoveredSquares = 0;

		this.revealAll = false;

		this._board = this.getInitialBoard(this.height, this.width, this.minesIndices);
		this.minesSquares = this.minesIndices.map(function (indices) {
			return _this._board[indices.y][indices.x];
		});
	}

	_createClass(Board, [{
		key: "discoverAndGetAllMines",
		value: function discoverAndGetAllMines() {
			var mines = this.minesSquares;
			var minesViews = [];
			for (var i = 0; i < mines.length; i++) {
				var mineView = this.getSquareView(mines[i]);
				mineView.isMine = true;
				minesViews[i] = mineView;
			}
			var result = minesViews;

			return result;
		}
	}, {
		key: "revealAllMines",
		value: function revealAllMines() {
			this.revealAll = true;
		}
	}, {
		key: "getSquare",
		value: function getSquare(y, x) {
			this.validateIndices(y, x);

			return this._board[y][x];
		}
	}, {
		key: "print",
		value: function print() {
			for (var i = 0; i < this.height; i++) {
				var s = "|";
				for (var j = 0; j < this.width; j++) {
					var temp = this._board[i][j];
					if (temp.hiddenData.isMine) {
						s += "x|";
					} else {
						s += temp.hiddenData.nearbyMines + "|";
					}
				}
				debug(s);
			}
		}
	}, {
		key: "getSettings",
		value: function getSettings() {
			return {
				height: this.height,
				width: this.width,
				minesCount: this.minesCount
			};
		}
	}, {
		key: "getView",
		value: function getView() {
			var result = [];
			for (var i = 0; i < this.height; i++) {
				result[i] = [];
				for (var j = 0; j < this.width; j++) {
					var s = this._board[i][j];
					var view = this.getSquareView(s);
					result[i][j] = view;
				}
			}
			return result;
		}
	}, {
		key: "toggleFlag",
		value: function toggleFlag(y, x) {
			var _this2 = this;

			this.validateIndices(y, x);

			var square = this.getSquare(y, x);
			if (square.exposedData.isDiscovered === true) {
				throw "square is already discovered";
			}
			debug("flagging  y=" + y + ", x=" + x);

			if (square.exposedData.isFlagged === false) {
				square.exposedData.isFlagged = true;
				this.flagsIndices.push({ x: square.location.x, y: square.location.y });
				this.flagsCount = this.flagsCount + 1;
				this.incrementNeighboursNearbyFlags(y, x);
			} else {
				square.exposedData.isFlagged = false;
				this.flagsIndices = this.flagsIndices.filter(function (indices) {
					return indices.x === square.location.x && indices.y === square.location.y;
				});
				this.flagsCount = this.flagsCount - 1;
				this.decreaseNeighboursNearbyFlags(y, x);
			}

			var result = {
				clickedSquare: this.getSquareView(square),
				neighbours: square.neighbours.map(function (square) {
					return _this2.getSquareView(square);
				})
			};

			return result;
		}
	}, {
		key: "decreaseNeighboursNearbyFlags",
		value: function decreaseNeighboursNearbyFlags(y, x) {
			this.updateNeighboursNearbyFlags(y, x, -1);
		}
	}, {
		key: "incrementNeighboursNearbyFlags",
		value: function incrementNeighboursNearbyFlags(y, x) {
			this.updateNeighboursNearbyFlags(y, x, 1);
		}
	}, {
		key: "updateNeighboursNearbyFlags",
		value: function updateNeighboursNearbyFlags(y, x, diff) {
			var square = this.getSquare(y, x);

			var _iteratorNormalCompletion = true;
			var _didIteratorError = false;
			var _iteratorError = undefined;

			try {
				for (var _iterator = square.neighbours[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
					var neighbour = _step.value;

					neighbour.exposedData.nearbyFlags += diff;
				}
			} catch (err) {
				_didIteratorError = true;
				_iteratorError = err;
			} finally {
				try {
					if (!_iteratorNormalCompletion && _iterator.return) {
						_iterator.return();
					}
				} finally {
					if (_didIteratorError) {
						throw _iteratorError;
					}
				}
			}
		}
	}, {
		key: "discover",
		value: function discover(y, x) {
			var _this3 = this;

			this.validateIndices(y, x);
			var square = this.getSquare(y, x);
			if (square.exposedData.isFlagged === true) {
				throw "square is already flagged";
			}

			debug("discovering for y=" + y + ", x=" + x + ", totalSquares=" + this.totalSquares + ", discoveredSquares=" + this.isDiscovered);
			var newlyDiscovered = [];
			square = this.recursiveDiscover(square, newlyDiscovered);
			debug("discovered for y=" + y + ", x=" + x + ", totalSquares=" + this.totalSquares + ", discoveredSquares=" + this.isDiscovered);

			var newlyDiscoveredView = newlyDiscovered.map(function (square) {
				return _this3.getSquareView(square);
			});
			var clickedSquare = this.getSquareView(square);

			var result = {
				newlyDiscovered: newlyDiscoveredView,
				clickedSquare: clickedSquare
			};

			return result;
		}
	}, {
		key: "multiDiscover",
		value: function multiDiscover(y, x) {
			var _this4 = this;

			this.validateIndices(y, x);
			var square = this.getSquare(y, x);
			if (square.exposedData.isFlagged === true) {
				throw "square is already flagged";
			} else if (square.exposedData.isDiscovered === false) {
				throw "square is not discovered yet";
			}
			var neighboursToDiscoverResult = this.getNeighboursToDiscover(y, x);
			if (square.hiddenData.nearbyMines !== neighboursToDiscoverResult.flagsCount || neighboursToDiscoverResult.neighboursToDiscover.length === 0) {
				throw "square is not satisfied";
			}
			var newlyDiscovered = [];
			for (var i = 0; i < neighboursToDiscoverResult.neighboursToDiscover.length; i++) {
				var currSquare = neighboursToDiscoverResult.neighboursToDiscover[i];
				var discoverResult = this.discover(currSquare.location.y, currSquare.location.x);
				newlyDiscovered = newlyDiscovered.concat(discoverResult.newlyDiscovered);
			}

			var newlyDiscoveredNeighboursView = neighboursToDiscoverResult.neighboursToDiscover.map(function (square) {
				return _this4.getSquareView(square);
			});
			var clickedSquareView = this.getSquareView(square);

			var result = {
				newlyDiscovered: newlyDiscovered,
				newlyDiscoveredNeighbours: newlyDiscoveredNeighboursView,
				clickedSquare: clickedSquareView
			};

			return result;
		}
	}, {
		key: "getNeighboursToDiscover",
		value: function getNeighboursToDiscover(y, x) {
			var neighbours = this.getSquare(y, x).neighbours;
			var neighboursToDiscover = [];

			var flagsCount = 0;
			var discoveredCount = 0;

			for (var i = 0; i < neighbours.length; i++) {
				var neighbour = neighbours[i];
				if (neighbour.exposedData.isDiscovered) {
					discoveredCount += 1;
				} else if (neighbour.exposedData.isFlagged) {
					flagsCount += 1;
				} else {
					neighboursToDiscover.push(neighbour);
				}
			}

			return { flagsCount: flagsCount, discoveredCount: discoveredCount, neighboursToDiscover: neighboursToDiscover };
		}
	}, {
		key: "recursiveDiscover",
		value: function recursiveDiscover(square, discoveredSoFar) {
			if (square.exposedData.isDiscovered === false && square.exposedData.isFlagged === false) {
				square.exposedData.isDiscovered = true;
				discoveredSoFar.push(square);
				this.discoveredSquares += 1;

				if (square.hiddenData.nearbyMines === 0 && square.hiddenData.isMine === false) {

					for (var i = 0; i < square.neighbours.length; i++) {
						this.recursiveDiscover(square.neighbours[i], discoveredSoFar); //calling recursive discover for all neighbors in case there are no mines nearby
					}
				}
			}
			return square;
		}
	}, {
		key: "validateIndices",
		value: function validateIndices(y, x) {
			if (!this.isValidIndices(y, x)) {
				throw "indexes does not exists on board. y=" + y + ",x=" + x;
			}
		}
	}, {
		key: "isValidIndices",
		value: function isValidIndices(y, x) {
			var b = this._board;
			if (y >= this.height || x >= this.width || y < 0 || x < 0) {
				return false;
			}
			return true;
		}
	}, {
		key: "getNeighbors",
		value: function getNeighbors(y, x, board) {

			var b = board;
			var neighbours = [];
			this.optAddNeighbor(neighbours, y, x + 1, board);
			this.optAddNeighbor(neighbours, y, x - 1, board);

			this.optAddNeighbor(neighbours, y + 1, x + 1, board);
			this.optAddNeighbor(neighbours, y + 1, x, board);
			this.optAddNeighbor(neighbours, y + 1, x - 1, board);

			this.optAddNeighbor(neighbours, y - 1, x + 1, board);
			this.optAddNeighbor(neighbours, y - 1, x, board);
			this.optAddNeighbor(neighbours, y - 1, x - 1, board);

			return neighbours;
		}
	}, {
		key: "optAddNeighbor",
		value: function optAddNeighbor(neighbours, y, x, board) {
			if (this.isValidIndices(y, x)) {
				neighbours.push(board[y][x]);
			}
		}
	}, {
		key: "calculateNearbyMines",
		value: function calculateNearbyMines(y, x, neighbours) {
			var nearbyMines = neighbours.filter(function (square) {
				return square.hiddenData.isMine === true;
			}).length;

			return nearbyMines;
		}
	}, {
		key: "getEmptyBoard",
		value: function getEmptyBoard(height, width) {
			var b = [];
			for (var i = 0; i < height; i++) {
				b[i] = [];
			}

			return b;
		}
	}, {
		key: "setMines",
		value: function setMines(height, width, minesIndices, board) {
			var b = board;
			//set the mines according to minesIndices
			for (var i = 0; i < minesIndices.length; i++) {
				var MineLocation = minesIndices[i];
				var mineSquare = new Square();
				mineSquare.location.x = MineLocation.x;
				mineSquare.location.y = MineLocation.y;
				mineSquare.hiddenData.isMine = true;
				mineSquare.hiddenData.nearbyMines = -1;
				b[MineLocation.y][MineLocation.x] = mineSquare;
			}

			return b;
		}
	}, {
		key: "initBlankSquares",
		value: function initBlankSquares(height, width, board) {
			var b = board;
			for (var i = 0; i < height; i++) {
				for (var j = 0; j < width; j++) {
					//mines are already defined and therefore the condition will be false for them.
					if (typeof b[i][j] === "undefined") {
						var s = new Square();
						s.location.y = i;
						s.location.x = j;
						b[i][j] = s;
					}
				}
			}

			return b;
		}
	}, {
		key: "initSquaresNeightbours",
		value: function initSquaresNeightbours(height, width, board) {
			var b = board;

			for (var i = 0; i < height; i++) {
				for (var j = 0; j < width; j++) {
					var s = b[i][j];
					s.neighbours = this.getNeighbors(i, j, board);
					if (s.hiddenData.isMine === false) {
						s.hiddenData.nearbyMines = this.calculateNearbyMines(i, j, s.neighbours);
					}
					b[i][j] = s;
				}
			}

			return b;
		}
	}, {
		key: "getInitialBoard",
		value: function getInitialBoard(height, width, minesIndices) {
			var board = this.getEmptyBoard(height, width);

			board = this.setMines(height, width, minesIndices, board);

			board = this.initBlankSquares(height, width, board);

			board = this.initSquaresNeightbours(height, width, board);

			return board;
		}
	}, {
		key: "getSquareView",
		value: function getSquareView(square) {
			var view = {
				isFlagged: square.exposedData.isFlagged,
				isDiscovered: square.exposedData.isDiscovered,
				nearbyFlags: square.exposedData.nearbyFlags,
				y: square.location.y,
				x: square.location.x
			};
			if (square.exposedData.isDiscovered === true || this.revealAll === true) {
				view.isMine = square.hiddenData.isMine;
				view.nearbyMines = square.hiddenData.nearbyMines;
			}
			return view;
		}
	}]);

	return Board;
}();

var _exports = module.exports = Board;