"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Board = require("./board");
var Game = require("./game");
var RandomBoardLocationsProvider = require("./randomBoardLocationsProvider");

var gameService = function () {
    function gameService(host) {
        _classCallCheck(this, gameService);

        this.games = {};
    }

    _createClass(gameService, [{
        key: "getGame",
        value: function getGame(gameId) {
            var game = this.games[gameId];
            if (game === undefined) {
                return "game " + gameId + " does not exist";
            } else {
                return Promise.resolve(game.getView());
            }
        }
    }, {
        key: "createGame",
        value: function createGame(height, width, mines) {
            var minesIndices = RandomBoardLocationsProvider.getRandomIndexesByBoardSize(height, width, mines);
            var board = new Board(height, width, minesIndices);

            var game = new Game(board);
            var id = this.generateId();
            this.games[id] = game;
            return Promise.resolve(id);
        }
    }, {
        key: "discoverAndGetAllMines",
        value: function discoverAndGetAllMines(gameId) {
            var game = this.games[gameId];
            if (typeof game === "undefined") {
                throw "game with the specified id:" + gameId + " is not available";
            } else if (game.state.isFinished === false) {
                throw "getAllBomb call is valid only when the game is finished";
            }

            var result = { mines: game.discoverAndGetAllMines() };

            return Promise.resolve(result);
        }
    }, {
        key: "discover",
        value: function discover(gameId, y, x) {
            var game = this.games[gameId];
            if (typeof game === "undefined") {
                throw "game with the specified id:" + gameId + " is not available";
            }
            var result = game.discover(y, x);

            return Promise.resolve(result);
        }
    }, {
        key: "multiDiscover",
        value: function multiDiscover(gameId, y, x) {
            var game = this.games[gameId];
            if (typeof game === "undefined") {
                throw "game with the specified id:" + gameId + " is not available";
            }
            var result = game.multiDiscover(y, x);

            return Promise.resolve(result);
        }
    }, {
        key: "toggleFlag",
        value: function toggleFlag(gameId, y, x) {
            var game = this.games[gameId];
            if (typeof game === "undefined") {
                throw "game with the specified id:" + gameId + " is not available";
            }
            var result = game.toggleFlag(y, x);

            return Promise.resolve(result);
        }
    }, {
        key: "generateId",
        value: function generateId() {
            return Math.random().toString(36).substr(2, 16);
        }
    }, {
        key: "handleError",
        value: function handleError(logMessage, error) {
            console.log(logMessage);
            console.log(error);
            throw error;
        }
    }]);

    return gameService;
}();

module.exports = gameService;